
float getChroma(vec3 col, int space)
{

  float chroma;
	if(space == 0)//YCC-HSY
	{
		float C1 = dot(col, vec3(0.701, -0.587, -0.114));
		float C2 = dot(col, vec3(-0.299, -0.587, 0.886));
		chroma = sqrt(C1*C1 + C2*C2);
	}
	else if(space == 1)//HSL
	{
		
		float cmax = max(max(col.r, col.g), col.b);
		float cmin = min(min(col.r, col.g), col.b);
		float lum = (cmin + cmax) / 2.0;
		if(lum <= 0.5) chroma = (cmax - cmin) / (cmax + cmin);
		else chroma = (cmax - cmin) / (2.0 - cmax - cmin);
	}
	else//HSV
	{
		float cmax = max(max(col.r, col.g), col.b);
		float cmin = min(min(col.r, col.g), col.b);
		if(cmax == 0.0) chroma = 0.0;
		else chroma = (cmax - cmin) / cmax;
	}
	return chroma;
}

uniform sampler2D image;
uniform int imgWidth, imgHeight;
uniform int type;//0:PrewittC1:SobelC2:RobertsC3:Laplacian
uniform int  inverse, binary, grad, space;
uniform float threshold;
varying vec2 pos;//eNX`W
//eNX`WԂɂfʒu̒P
float dx = 1.0 / float(imgWidth);
float dy = 1.0 / float(imgHeight);

void main(void)
{
	int i, j, k, l;
	vec2 weight[9];
	vec2 offset[9];
	//offseťvZ
	for(int j = 0; j < 3; j++)
		for(int i = 0; i < 3; i++)
		{ 
			offset[i + j * 3] = vec2(float(-1 + i) * dx, float(-1 + j) * dy) ;
		}

	if(type == 0)
	{
		//Prewitt
		//X
		weight[0].x = -1.0; weight[1].x = 0.0; weight[2].x = 1.0;
		weight[3].x = -1.0; weight[4].x = 0.0; weight[5].x = 1.0;
		weight[6].x = -1.0; weight[7].x = 0.0; weight[8].x = 1.0;
		//Y
		weight[0].y = -1.0; weight[1].y =-1.0; weight[2].y =-1.0;
		weight[3].y =  0.0; weight[4].y = 0.0; weight[5].y = 0.0;
		weight[6].y =  1.0; weight[7].y = 1.0; weight[8].y = 1.0;		
	}

	else if(type == 1)
	{
		//Sobel
		//X
		weight[0].x = -1.0; weight[1].x = 0.0; weight[2].x = 1.0;
		weight[3].x = -2.0; weight[4].x = 0.0; weight[5].x = 2.0;
		weight[6].x = -1.0; weight[7].x = 0.0; weight[8].x = 1.0;
		//Y
		weight[0].y = -1.0; weight[1].y =-2.0; weight[2].y =-1.0;
		weight[3].y =  0.0; weight[4].y = 0.0; weight[5].y = 0.0;
		weight[6].y =  1.0; weight[7].y = 2.0; weight[8].y = 1.0;
	}
	else if(type == 2)
	{
		//Roberts
		//X
		weight[0].x = 0.0; weight[1].x = 0.0; weight[2].x = 0.0;
		weight[3].x = 0.0; weight[4].x = 0.0; weight[5].x = 1.0;
		weight[6].x = 0.0; weight[7].x =-1.0; weight[8].x = 0.0;
		//Y
		weight[0].y = 0.0; weight[1].y = 0.0; weight[2].y = 0.0;
		weight[3].y = 0.0; weight[4].y = 1.0; weight[5].y = 0.0;
		weight[6].y = 0.0; weight[7].y = 0.0; weight[8].y =-1.0;		
	}
	else 
	{
		//Laplacian
		weight[0].x = 0.0; weight[1].x = 1.0; weight[2].x = 0.0;
		weight[3].x = 1.0; weight[4].x =-4.0; weight[5].x = 1.0;
		weight[6].x = 0.0; weight[7].x = 1.0; weight[8].x = 0.0;
	}


	vec2 texCoord = vec2(pos.x * dx, pos.y * dy);
	vec4 col4;
	vec4 colX = vec4(0.0);
	vec4 colY = vec4(0.0);
	vec3 col = vec3(0.0);
	float chroma;
	//ݍ݉Z
	if(type < 3)//1
	{
		for(j = 0; j < 9; j++) 
		{
			col4 = texture2D(image, texCoord + offset[j]);
			chroma = getChroma(col4.rgb, space);
			colX += chroma * weight[j].x;
			colY += chroma * weight[j].y;
		}
		if(grad == 0)
			col = sqrt(colX*colX + colY*colY).rgb;
		else if(grad == 1)
			col = abs(colX).rgb + abs(colY).rgb;
 		else col = max(abs(colX), abs(colY)).rgb;
	}
	else//Laplacian
	{
		for(j = 0; j < 9; j++) 
		{
			col4 = texture2D(image, texCoord + offset[j]);
			chroma = getChroma(col4.rgb, space);
			col += chroma * weight[j].x;
		}
		col += vec3(0.5);//Ԓlǉ
	}
	
//	if(gray == 1) //grayscale
		//col = vec3((col.r + col.g + col.b) / 3.0);
//		col = vec3(dot(col, vec3(0.299, 0.587, 0.114))); 
	if(inverse == 1)//] 
		col = vec3(1.0) - col;
	if(binary == 1)//2l 
		col = step(threshold, col);
	
	gl_FragColor = vec4(vec3(col), 1.0);	
}
