
vec3 RGB2HSV(vec3 col)
{
	float hue, chroma, value;
	float cmax = max(max(col.r, col.g), col.b);
	float cmin = min(min(col.r, col.g), col.b);
	//chroma
	if(cmax == 0.0) chroma = 0.0;
	else chroma = (cmax - cmin) / cmax;
	//hue
	if(cmax == cmin) hue = 0.0;
	else if(cmax == col.r) {
		hue = (col.g - col.b) / (cmax - cmin);
		if(hue < 0.0) hue += 6.0;
	}
	else if(cmax == col.g) 
	  hue = (col.b - col.r) / (cmax - cmin) + 2.0;
	else 
	  hue = (col.r - col.g) / (cmax - cmin) + 4.0;
	hue *= 60.0;
	//value
  value = cmax;
  vec3 hsv = vec3(hue, chroma, value);
  return hsv;
}

vec3 HSV2RGB(vec3 hsv)
{
	vec3 col;
	float hue = hsv.r;
	float chroma = hsv.g;
	float value = hsv.b;
	if(chroma == 0.0) return(vec3(hsv.b));
	int h = int(floor(hue / 60.0));
	float f = hue / 60.0 - float(h);
	float p = value * (1.0 - chroma);
	float q = value * (1.0 - f * chroma);
	float r = value * (1.0 - (1.0 - f) * chroma);
	if(h == 0) col = vec3(value, r, p);
	else if(h == 1) col = vec3(q, value, p);
	else if(h == 2) col = vec3(p, value, r);
	else if(h == 3) col = vec3(p, q, value);
	else if(h == 4) col = vec3(r, p, value);
	else col = vec3(value, p, q);
	return col;
} 

uniform sampler2D image;
uniform int imgWidth, imgHeight;
uniform float deltaHue, deltaChroma, deltaValue;
varying vec2 pos;//eNX`W
//eNX`WԂɂfʒu̒P
float dx = 1.0 / float(imgWidth);
float dy = 1.0 / float(imgHeight);

void main(void)
{
	int i, j, k, l;
	vec2 texCoord = vec2(pos.x * dx, pos.y * dy);
	vec3 col = texture2D(image, texCoord).rgb;
	vec3 hsv = RGB2HSV(col);//RGB->HSVϊ
	//new Hue
	hsv.r += deltaHue;
	if(hsv.r >= 360.0) hsv.r -= 360.0;
	if(hsv.r < 0.0)   hsv.r += 360.0;
	//new Chroma
	hsv.g += deltaChroma;
	hsv.g = clamp(hsv.g, 0.0, 1.0);
	//new Value
	hsv.b += deltaValue;
	hsv.b = clamp(hsv.b, 0.0, 1.0);
	col = HSV2RGB(hsv);//HSV->RGBϊ
	gl_FragColor = vec4(col, 1.0);		
}
