
vec3 RGB2HSV(vec3 col)
{
	float hue, chroma, value;
	float cmax = max(max(col.r, col.g), col.b);
	float cmin = min(min(col.r, col.g), col.b);
	//chroma
	if(cmax == 0.0) chroma = 0.0;
	else chroma = (cmax - cmin) / cmax;
	//hue
	if(cmax == cmin) hue = 0.0;
	else if(cmax == col.r) {
		hue = (col.g - col.b) / (cmax - cmin);
		if(hue < 0.0) hue += 6.0;
	}
	else if(cmax == col.g) hue = (col.b - col.r) / (cmax - cmin) + 2.0;
	else hue = (col.r - col.g) / (cmax - cmin) + 4.0;
	hue *= 60.0;
	//value
  value = cmax;
  vec3 hsv = vec3(hue, chroma, value);
  return hsv;
}

vec3 HSV2RGB(vec3 hsv)
{
	vec3 col;
	float hue = hsv.r;
	float chroma = hsv.g;
	float value = hsv.b;
	if(chroma == 0.0) return(vec3(hsv.b));
	int h = int(floor(hue / 60.0));
	float f = hue / 60.0 - float(h);
	float p = value * (1.0 - chroma);
	float q = value * (1.0 - f * chroma);
	float r = value * (1.0 - (1.0 - f) * chroma);
	if(h == 0) col = vec3(value, r, p);
	else if(h == 1) col = vec3(q, value, p);
	else if(h == 2) col = vec3(p, value, r);
	else if(h == 3) col = vec3(p, q, value);
	else if(h == 4) col = vec3(r, p, value);
	else col = vec3(value, p, q);
	return col;
} 
//---------------------------------------------
uniform sampler2D image;
uniform int imgWidth, imgHeight;
uniform int nLevelH, nLevelS, nLevelV;
uniform float hue, chroma, value, gamma;
uniform float threshEdge;
uniform int modeRender;
varying vec2 pos;//eNX`W
//eNX`WԂɂfʒu̒P
float dx = 1.0 / float(imgWidth);
float dy = 1.0 / float(imgHeight);
const float maxV = 255.0;//Kx̍ől
const vec3 minCol = vec3(0.5, 0.3, 0.3);//Zxŏl
//const vec3 minCol = vec3(0.2, 0.2, 0.2);//Zxŏl

void main(void)
{
	float kk;
	float hStep, sStep, vStep;
	vec2 texCoord = vec2(pos.x * dx, pos.y * dy);
	vec3 col0 = texture2D(image, texCoord).rgb;

	//Px
	col0 = pow(col0, vec3(gamma));

	//HSVɂK팸
	//RGB->HSVϊ
	vec3 hsv = RGB2HSV(col0) * maxV;
	if(nLevelH == 1) hsv.r = hue;//FŒl
	else
	{
		hStep = maxV / float(nLevelH-1);//FXebv
		//new Hue
		hsv.r /= 360.0;//[0, 1]͈̔͂ɕϊ
		kk = floor(hsv.r / hStep + 0.5);
		hsv.r = hStep * kk * 360.0 / maxV;
		if(hsv.r >= 360.0) hsv.r -= 360.0;
		if(hsv.r < 0.0)   hsv.r += 360.0;
	}
	if(nLevelS == 1) hsv.g = chroma;//ʓxŒl
	else
	{
		sStep = maxV / float(nLevelS-1);//ʓxXebv
		//new Chroma
		kk = floor(hsv.g / sStep + 0.5);
		hsv.g = sStep * kk / maxV;
	}
	if(nLevelV == 1) hsv.b = value;//xŒl
	else
	{	
		vStep = maxV / float(nLevelV-1);//xXebv	
		//new Value
		kk = floor(hsv.b / vStep + 0.5);
		hsv.b = vStep * kk / maxV;
	}
	//HSV->RGBϊ
	col0 = HSV2RGB(hsv);
	col0 = (vec3(1.0) - minCol) * col0 + minCol;;
	
	//Prewitt filter ɂtB^
	int i, j, k;
  int size = 3;;
	vec2 weight[9];
	vec2 offset[9];
	//offseťvZ
	for(int j = 0; j < size; j++)
		for(int i = 0; i < size; i++)
		{ 
			offset[i + j * size] = vec2(float(-1 + i) * dx, float(-1 + j) * dy) ;
		}

	//Prewitt Operator
	//X
	weight[0].x = -1.0; weight[1].x = 0.0; weight[2].x = 1.0;
	weight[3].x = -1.0; weight[4].x = 0.0; weight[5].x = 1.0;
	weight[6].x = -1.0; weight[7].x = 0.0; weight[8].x = 1.0;
	//Y
	weight[0].y = -1.0; weight[1].y =-1.0; weight[2].y =-1.0;
	weight[3].y =  0.0; weight[4].y = 0.0; weight[5].y = 0.0;
	weight[6].y =  1.0; weight[7].y = 1.0; weight[8].y = 1.0;		

	//l
	dx = 0.0;
	dy = 0.0;
	float dd, z;
	vec3 colP = vec3(0.0);
	//ݍ݉Z
	for(j = 0; j < size; j++) 
		for(i = 0; i < size; i++) {
			k = i+j*size;
			colP = texture2D(image, texCoord + offset[k]).rgb;
		  z = RGB2HSV(colP).b;//x
			dx += z * weight[k].x;
			dy += z * weight[k].y;
		}
		dd = sqrt(dx*dx + dy*dy);

  colP = vec3(dd);
	//] 
	colP = vec3(1.0) - colP;
	//2l 
	colP = clamp(step(threshEdge, colP), minCol, vec3(1.0, 1.0, 1.0));

	//Z
	vec3 col = col0 * colP;
	
	if(modeRender == 0) gl_FragColor = vec4(col, 1.0);	
	else if(modeRender == 1) gl_FragColor = vec4(col0, 1.0);	
	else gl_FragColor = vec4(colP, 1.0);
}
